OPTION ARB_precision_hint_nicest;

TEMP	light, color, R1, R2, localNormal, specular;

#PARAM   subOne = { -1, -1, -1, -1 };
#PARAM   scaleTwo = { 2, 2, 2, 2 };
#PARAM   subOne = { -1.0625, -1.0625, -1.0625, -1.0625 };
#PARAM   scaleTwo = { 2.125, 2.125, 2.125, 2.125 };
#PARAM   subOne = { -1.03125, -1.03125, -1.03125, -1.03125 };
#PARAM   scaleTwo = { 2.0625, 2.0625, 2.0625, 2.0625 };
PARAM   subOne = { -1.015625, -1.015625, -1.015625, -1.015625 };
PARAM   scaleTwo = { 2.03125, 2.03125, 2.03125, 2.03125 };

PARAM	grayscale = { 0.3, 0.59, 0.11, 0.0};

# load the specular half angle first, because
# the ATI shader gives a "too many indirections" error
# if this is done right before the texture indirection

#-----------------
$if !r_shaderPreferALU
TXB	specular, fragment.texcoord[6], $normalCubeMap, CUBE;
MAD	specular, specular, scaleTwo, subOne;
$else
# instead of using the normalization cube map, normalize with math
DP3		specular.x, fragment.texcoord[6], fragment.texcoord[6];
RSQ		specular.x, specular.x;
MUL		specular, specular.x, fragment.texcoord[6];
$endif
#-----------------

#$ifdef 	parallax
	TEMP newtexcoord;

	# load heightmap
	TXB		R1, fragment.texcoord[1], $heightMap, 2D;	#texture[7], 2D;
	MAD		R1, R1, program.local[0].x, program.local[0].y; # scale and bias
	MAD		newtexcoord, R1, specular, fragment.texcoord[1];

	TEMP 	height, viewofs, _bumpTC, _diffTC, _specTC, _lookTC;

	DP3		viewofs.w, fragment.texcoord[7], fragment.texcoord[7];
	#MUL 	viewofs.xyzw, fragment.texcoord[7], fragment.texcoord[7];
	RSQ 	viewofs.w, viewofs.w;
	MUL 	viewofs.xyzw, viewofs.w, fragment.texcoord[7];

#	TXB	 height, newtexcoord, $heightMap, 2D;
##	TXB	 height, newtexcoord, $bumpMap, 2D;
	TXB	 height, fragment.texcoord[4], $heightMap, 2D;
#	TXB	 height, fragment.texcoord[4], $bumpMap, 2D;
	#MUL 	height.x, height.x, -0.0005;
	#MUL 	height.z, height.z, 0.0005;
	MAD		height.x, height.z, -0.0005, 0.0005;	#, -0.0525, 0.0525;	#-0.0425, 0.0425;	#-0.0375, 0.0375;

	MAD _bumpTC, viewofs, height.x, newtexcoord;			#fragment.texcoord[1];
	MAD _diffTC, viewofs, height.x, fragment.texcoord[4];
	MAD _specTC, viewofs, height.x, fragment.texcoord[5];
	MAD _lookTC, viewofs, height.x, specular;				#fragment.texcoord[6];
#$else
#	ATTRIB _bumpTC = newtexcoord;							#fragment.texcoord[1];
#	ATTRIB _diffTC = fragment.texcoord[4];
#	ATTRIB _specTC = fragment.texcoord[5];
#	ATTRIB _lookTC = specular;								#fragment.texcoord[6];
#$endif

$ifdef enable_instdither
MUL R1.xy, fragment.position, 0.0625;
TXB R1, R1, $noiseMap, 2D;
SLT R1.w, _lookTC.w, R1.w;
KIL	-R1.w;
$endif


#
# the amount of light contacting the fragment is the
# product of the two light projections and the surface
# bump mapping
#

# perform the diffuse bump mapping

#-----------------
$if !r_shaderPreferALU
TXB	light, fragment.texcoord[0], $normalCubeMap, CUBE;
MAD	light, light, scaleTwo, subOne;
$else
# instead of using the normalization cube map, normalize with math
DP3		light.x, fragment.texcoord[0], fragment.texcoord[0];
RSQ		light, light.x;
MUL		light, light.x, fragment.texcoord[0];
$endif
#-----------------

TXB	localNormal, newtexcoord, $bumpMap, 2D;
$if !r_dxnNormalMaps
MOV localNormal.x, localNormal.a;	#com
#MOV   localNormal.z, 0.90;	#0.99;
$endif
MAD	localNormal.xyz, localNormal, scaleTwo, subOne;
DP3   localNormal.w, localNormal, localNormal;
RSQ   localNormal.w, localNormal.w;
MUL   localNormal.xyz, localNormal, localNormal.w;
MUL localNormal.zw, localNormal.xyxy, localNormal.xyxy;

ADD_SAT localNormal.z, localNormal, localNormal.w;
ADD localNormal.z, -localNormal, 1.0;
RSQ localNormal.z, localNormal.z;
RCP localNormal.z, localNormal.z;

# localNormal filter
MOV		R1.xyz, localNormal;
MOV		R1.z, ZeroFive;
DP3		R1, R1, R1;
MAD_SAT	R1.z, R1.w, -1, 1;
RSQ		R1, R1.z;
RCP		localNormal.z, R1.w;

$if r_normalizeNormalMaps
#MOV		localNormal.z, 0;
#DP3		R1.x, localNormal,localNormal;
#ADD		R1.x, 1, -R1.x;
#RSQ		R1.x, R1.x;
#RCP		localNormal.z, R1.x;
$endif

# localNormal filter
ADD_SAT	R1, newtexcoord, 0;
TXB		R2, R1, $bumpMap, 2D;		#texture[1], 2D;
MOV 	R2.x, R2.a;
MAD_SAT	R2, R2, scaleTwo, subOne;

PARAM   scaleTwoX = { 4.03125, 4.03125, 4.03125, 4.03125 };
PARAM   subOneX = { -3.015625, -3.015625, -3.015625, -3.015625 };

# flatten normal over distance (Z)
MAD_SAT	R1, fragment.position.z, scaleTwoX, subOneX;
MAX		R1, R1.x, 0;
ADD_SAT	localNormal.z, localNormal.z, R1.x;

DP3	light, light, localNormal;
MOV	light, 1;

# boost projection, specular light reaches further - DFM added
#ADD_SAT	R1.xyz, R1, R1;
#MUL	R2.rgb, R2, R1;

# modulate by the light falloff
#TXB	R1, fragment.texcoord[2], $lightFalloffMap, 2D;
#MUL	light, light, R1;

# compute "length" from diffuse falloff texcoords
SUB	R1.xy, fragment.texcoord[2], 0.375;		#0.7;
MUL	R1.xy, R1, R1;
ADD	R1.w, R1.x, R1.y;
RSQ	R1.w, R1.w;
RCP	R1.w, R1.w;
MUL	R1.w, R1, 0.75;							#0.7;

# modulate by "lenght"
SUB_SAT	light.xyz, light, R1.w;
SUB_SAT	R2.rgb, R2, R1.w;

# modulate by the light projection
TXP	R1, fragment.texcoord[3], $lightProjectionMap, 2D;
MUL	light, light, R1;

#
# the light will be modulated by the diffuse and
# specular surface characteristics
#

# modulate by the diffuse map and constant diffuse factor
TEMP diffuse;
TXB	diffuse, _diffTC, $diffuseMap, 2D;
MUL	color, diffuse, $diffuseColor;
$ifdef alphatest_kill
TEMP	_AR0;
SUB		_AR0.a, color.a, $alphaThresh;
KIL		_AR0.a;
$endif
$ifdef alphatocoverage
TEMP	_AR0;
SUB		_AR0.a, color.a, 0.1;
KIL		_AR0.a;
$endif

# perform the specular bump mapping
DP3	_lookTC, _lookTC, localNormal;
MOV _lookTC, _lookTC.z;

$if !r_shaderPreferALU
	# perform a dependent table read for the specular falloff
	TXB	R1, _lookTC, $specularTable, 2D;
$else
	# Specular falloff: (sat(4x-3))^2 matches the function inside the specular table texture
	#MAD_SAT R1.x, _lookTC.x, 4.0, -3.0;
	#PARAM specExp = { 16, 0, 0, 0 };
	PARAM	specExp = { 12, 0, 0, 0 };
	POW_SAT R1.x, _lookTC.x, specExp.x;
	#MUL R1.x, R1.x, R1.x;
$endif

# no color please!
ADD	R1, R1.x, R1;

# modulate by the constant specular factor
MUL	R1, R1.x, $specularColor;

# modulate by the specular map * 2
TXB	R2, _specTC, $specularMap, 2D;
#TXB	R2, newtexcoord, $specularMap, 2D;
#ADD	R2, R2, R2;

# modulate by the constant specular factor
#MUL R2.rgb, R2.x, $diffuseColor;

# no color please!
#ADD	R2.rgb, R2.x, R2;

MAD	color, R1, R2, color;
MUL	color, light, color;

# modify by the vertex color
MUL color, color, fragment.color;

$if ( r_megaDrawMethod != 0 )
$ifdef amblit
LRP result.color.rgb, fragment.color.secondary.x, $fogColor, color;
$else
SUB R1.x, 1, fragment.color.secondary.x;
MUL result.color.rgb, color, R1.x;
$endif
$else
MOV result.color.rgb, color;
$endif


MUL R1, R1, R2;
MUL R1, R1, light;
DP3 R1, R1, grayscale;
$ifdef alphatocoverage
MUL result.color.a, diffuse.a, 1.5;
$else
MOV result.color.a,  1;#diffuse;#R1;
$endif
# this should be better on future hardware, but current drivers make it slower
MUL result.color.xyz, color, fragment.color;

